@@ -457,22 +457,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setGenericSpct(x = xx,
-                       multiple.wl = multiple.wl,
-                       idfactor = idfactor)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -492,22 +485,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setRawSpct(x = xx,
-                   multiple.wl = multiple.wl,
-                   idfactor = idfactor)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -527,22 +513,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setCpsSpct(x = xx,
-                   multiple.wl = multiple.wl,
-                   idfactor = idfactor)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -562,25 +541,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setSourceSpct(x = xx,
-                      time.unit = getTimeUnit(x),
-                      bswf.used = getBSWFUsed(x),
-                      multiple.wl = multiple.wl,
-                      idfactor = idfactor,
-                      strict.range = NA_integer_)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -600,23 +569,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setResponseSpct(x = xx,
-                        time.unit = getTimeUnit(x),
-                        multiple.wl = multiple.wl,
-                        idfactor = idfactor)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -636,24 +597,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setFilterSpct(x = xx,
-                      Tfr.type = getTfrType(x),
-                      multiple.wl = multiple.wl,
-                      idfactor = idfactor,
-                      strict.range = NA_integer_)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -673,24 +625,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setReflectorSpct(x = xx,
-                         Rfr.type = getRfrType(x),
-                         multiple.wl = multiple.wl,
-                         idfactor = idfactor,
-                         strict.range = NA_integer_)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -710,25 +653,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setObjectSpct(x = xx,
-                      Tfr.type = getTfrType(x),
-                      Rfr.type = getRfrType(x),
-                      multiple.wl = multiple.wl,
-                      idfactor = idfactor,
-                      strict.range = NA_integer_)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
@@ -748,22 +681,15 @@ subset.generic_spct <- function(x, subset, select, drop = FALSE, ...) {
     }
     if (is.data.frame(xx)) {
       if ("w.length" %in% names(xx)) {
-        if (nrow(xx) == nrow(x)) {
-          multiple.wl <- getMultipleWl(x)
-        } else {
+        if (!(getMultipleWl(x) == 1L || nrow(xx) == nrow(x))) {
           # subsetting of rows can decrease the number of spectra
-          # even from one to zero!!
           multiple.wl <- findMultipleWl(xx, same.wls = FALSE)
+          xx <- setMultipleWl(xx, multiple.wl)
         }
-        idfactor <- getIdFactor(x)
-        if (is.na(idfactor)) {
-          idfactor <- NULL
+        if (ncol(xx) != ncol(x)) {
+          xx <- copy_attributes(x, xx)
         }
-        # that the ID variable has not been deleted is tested when set
-        setChromaSpct(xx,
-                      multiple.wl = multiple.wl,
-                      idfactor = idfactor)
-        # other attributes remain unchanged
+        xx <- check_spct(xx)
       } else {
         rmDerivedSpct(xx)
       }
